function [ M_new ] = error_generator( M )

measurement_units = M;

% -------- Measurement standard deviation input -------
sd_type_1 = 0.0027451*0.5;
sd_type_2 = 0.0027451*0.5;
sd_type_3 = 0.01;

pd1 = makedist('Normal','mu',0,'sigma',sd_type_1);
pd2 = makedist('Normal','mu',0,'sigma',sd_type_2);
pd3 = makedist('Normal','mu',0,'sigma',sd_type_3);

num_t_1 = 0; % Amount of type 1 measurements
num_t_2 = 0; % Amount of type 2 measurements
num_t_3 = 0; % Amount of type 3 measurements
for n = 1:length(measurement_units)
    if measurement_units(n,2) == 1      
        num_t_1 = num_t_1 + 1;
    elseif measurement_units(n,2) == 2
        num_t_2 = num_t_2 + 1;
    elseif measurement_units(n,2) == 100
        num_t_3 = num_t_3 + 1;
    end
end

for n = 1:num_t_1                               % Type 1
    pd1 = makedist('Normal','mu',0,'sigma',sd_type_1 * sqrt((real(measurement_units(n,5)))^2+(imag(measurement_units(n,5)))^2));
    measurement_units(n,5) =                    measurement_units(n,5) + random(pd1) + random(pd1)*1i;
end
for n = 1:num_t_2                               % Type 2
    pd2 = makedist('Normal','mu',0,'sigma',sd_type_2 * sqrt( (real(measurement_units(num_t_1+n,5)))^2 + (imag(measurement_units(num_t_1+n,5)))^2));
    measurement_units(num_t_1+n,5) =            measurement_units(num_t_1+n,5) + random(pd2) + random(pd2)*1i;
end
% for n = 1:num_t_3                               % Type 3
%     pd3 = makedist('Normal','mu',0,'sigma',sd_type_3 * sqrt((real(measurement_units(num_t_1+num_t_2+n,5)))^2+(imag(measurement_units(num_t_1+num_t_2+n,5)))^2));
%     measurement_units(num_t_1+num_t_2+n,5) =    measurement_units(num_t_1+num_t_2+n,5) + random(pd3) + random(pd3)*1i;
% end


M_new = measurement_units;




% ---------------- PRINTING FOR VALIATION -------------------

% measurement_units_unchanged = [
%     % Type 1: PMU bus voltage measurements
%     1 1 1 0 0.2865+1.0123i 10;
%     2 1 5 0 0.2111+1.0286i 10;
%     3 1 7 0 0.2128+1.0161i 10;
%     4 1 10 0 0.1372+1.0418i 10;
%     % Type 2: PMU branch current measurement
%     5 2 1 2 0.0032749+0.029299i 10;
%     6 2 1 3 -0.0076258-0.016859i 10;
%     7 2 5 4 0.00023574+0.039876i 10;
%     8 2 5 7 0.0037916+0.002947i 10;
%     9 2 5 6 0.0045808+0.0045069i 10;
%     10 2 7 5 -0.0076252-0.0021521i 10;
%     11 2 7 6 -0.0041018+0.0063759i 10;
%     12 2 7 8 -0.0012598-0.027938i 10;
%     13 2 10 9 -0.0034527+0.030852i 10;
%     % Type 3: Classic state estimation bus voltage value
%     14 3 1 0 0.2865+1.0123i 10;
%     15 3 2 0 0.3224+0.9981i 10;
%     16 3 3 0 0.2742+1.0206i 10;
%     17 3 4 0 0.2894+1.0046i 10;
%     18 3 5 0 0.2111+1.0286i 10;
%     19 3 6 0 0.2154+1.0168i 10;
%     20 3 7 0 0.2128+1.0161i 10;
%     21 3 8 0 0.1909+1.0249i 10;
%     22 3 9 0 0.1634+1.0343i 10;
%     23 3 10 0 0.1372+1.0418i 10;
%     ];




% %------ FOR PRINTING PERCENTAGE ERRORS IN MAGNITUDE AND ANGLE ---------
% 
% %MAGNITUDE
% 
% for n = 1:(num_t_1+num_t_2)
%     if n == 1
%         fprintf('Type 1: PMU Voltage \n');
%     end
%     actual = sqrt( (real(measurement_units_unchanged(n,5)))^2 + (imag(measurement_units_unchanged(n,5)))^2 );
%     measurement = sqrt( (real(measurement_units(n,5)))^2 + (imag(measurement_units(n,5)))^2 );
%     error = 100 * (measurement - actual) / actual;
%     st = ['Measurement vector ', num2str(n), ': ', num2str( measurement ) , 9 , 'Real vector: ', num2str( actual ) 9 'Diff: ' , num2str( error )];
%     disp(st);
% 
% 
%     if n == num_t_1
%         fprintf('Type 2: PMU Current \n');
%     end
% end
% % 
% % 
% % %ANGLE
% % 
% for n = 1:(num_t_1+num_t_2)
%     if n == 1
%         fprintf('Type 1: PMU Voltage \n');
%     end
%     actual = atan(imag(measurement_units_unchanged(n,5))/real(measurement_units_unchanged(n,5)))*360/(2*pi);
%     measurement = atan(imag(measurement_units(n,5))/real(measurement_units(n,5)))*360/(2*pi);
%     error = 100 * (measurement - actual) / actual;
%     st = ['Measurement vector ', num2str(n), ': ', num2str( measurement ) , 9 , 'Real vector: ', num2str( actual ) 9 'Diff: ' , num2str( error )];
%     disp(st);
% 
% 
%     if n == num_t_1
%         fprintf('Type 2: PMU Current \n');
%     end
% end

end

% % ---------------- FOR TESTING OF MEASUREMENT ERRORS ------------
% test = zeros(1000,1);
% for b=1:13
% for t = 1:1000;
% 
%     measurement_units(b,5) = measurement_units_unchanged(b,5);
% for n = b:b                               % Type 1
%     pd1 = makedist('Normal','mu',0,'sigma',sd_type_1 * sqrt((real(measurement_units(n,5)))^2+(imag(measurement_units(n,5)))^2));
%     measurement_units(n,5) =                    measurement_units(n,5) + random(pd1) + random(pd1)*1i;
% end
% for n = b:b                               % Type 2
%     pd2 = makedist('Normal','mu',0,'sigma',sd_type_2 * sqrt( (real(measurement_units(num_t_1+n,5)))^2 + (imag(measurement_units(num_t_1+n,5)))^2));
%     measurement_units(num_t_1+n,5) =            measurement_units(num_t_1+n,5) + random(pd2) + random(pd2)*1i;
% end
% 
% actual = sqrt( (real(measurement_units_unchanged(b,5)))^2 + (imag(measurement_units_unchanged(b,5)))^2 );
%     measurement = sqrt( (real(measurement_units(b,5)))^2 + (imag(measurement_units(b,5)))^2 );
%     error = 100 * (measurement - actual) / actual;
% test(t,1) = error;
% end
% test = sort(test);
% plot(test);
% hold on
% end