function [ H , W, M ] = reconstruction( branches , measurement_units )

% Finding the amount of each type of measurement
num_t_1 = 0; % Amount of type 1 measurements
num_t_2 = 0; % Amount of type 2 measurements
num_t_3 = 0; % Amount of type 3 measurements
for i = 1:length(measurement_units)
    if measurement_units(i,2) == 1
        num_t_1 = num_t_1 + 1;
    elseif measurement_units(i,2) == 2
        num_t_2 = num_t_2 + 1;
    elseif measurement_units(i,2) == 3
        num_t_3 = num_t_3 + 1;
    end
end



% Finding the amount of branches
num_branches = length(branches);

% Changing branch-matrix series values from impedance to admittance
for i = 1:num_branches
    branches(i,4) = 1/branches(i,4);
end

% Changing branch/matrix shunt values from the total admittance to half of
% the total admittance
for i = 1:num_branches
    branches(i,5) = 0.5*branches(i,5);
end


% Finding the amount of buses
% (should be the same as number of type 3 measurements)
high = max(branches);
num_buses = max(high(2),high(3));

% Creating empty H-matrix
H = zeros(2 * (num_t_1 + num_t_2 + num_t_3) , 2 * num_buses);

% Filling state estimator part of H-matrix
for i = 1:(2 * num_t_3)
    H(i,i) = 1;
end

% Filling PMU voltage measurement part of H-matrix
for i = 1:num_t_1
    H(2 * num_t_3 + 2 * i - 1, 2 * measurement_units(i,3) - 1) = 1;
    H(2 * num_t_3 + 2 * i, 2 * measurement_units(i,3)) = 1;
end

% Pairing PMU current measurement units with their relevant branchs
pairing = zeros(num_t_2,1);
for i = 1:num_t_2
    for n = 1:num_branches
        if( (measurement_units(num_t_1 + i,3) == branches(n,2)) && (measurement_units(num_t_1 + i,4) == branches(n,3)) || (measurement_units(num_t_1 + i,3) == branches(n,3)) && (measurement_units(num_t_1 + i,4) == branches(n,2)) )
            pairing(i) = branches(n,1);
        end
    end
end

% Filling PMU current measurement part of H-matrix
for i = 1:num_t_2
    H(2 * (num_t_1 + num_t_3) + 2 * i - 1, 2 * measurement_units(num_t_1 + i,3) - 1) = real(branches(pairing(i),4) + branches(pairing(i),5));
    
    H(2 * (num_t_1 + num_t_3) + 2 * i - 1, 2 * measurement_units(num_t_1 + i,3)) = - imag(branches(pairing(i),4) + branches(pairing(i),5));
    
    H(2 * (num_t_1 + num_t_3) + 2 * i - 1, 2 * measurement_units(num_t_1 + i,4) - 1) = - real(branches(pairing(i),4));
    
    H(2 * (num_t_1 + num_t_3) + 2 * i - 1, 2 * measurement_units(num_t_1 + i,4)) = imag(branches(pairing(i),4));
    
    H(2 * (num_t_1 + num_t_3) + 2 * i, 2 * measurement_units(num_t_1 + i,3) - 1) = imag(branches(pairing(i),4) + branches(pairing(i),5));
    
    H(2 * (num_t_1 + num_t_3) + 2 * i, 2 * measurement_units(num_t_1 + i,3)) = real(branches(pairing(i),4) + branches(pairing(i),5));
    
    H(2 * (num_t_1 + num_t_3) + 2 * i, 2 * measurement_units(num_t_1 + i,4) - 1) = - imag(branches(pairing(i),4));
    
    H(2 * (num_t_1 + num_t_3) + 2 * i, 2 * measurement_units(num_t_1 + i,4)) = - real(branches(pairing(i),4));

end


% Creating empty W-matrix
W = zeros(2 * ( num_t_1 + num_t_2 + num_t_3 ));

% Filling W-matrix
for i = 1:num_t_3 %Confidence for SE voltage values
    W(2*i-1,2*i-1) = measurement_units(num_t_1 + num_t_2 + i,6);
    W(2*i,2*i) = measurement_units(num_t_1 + num_t_2 + i,6);
end
for i = 1:num_t_1 %Confidence for PMU voltage values
    W(2*(num_t_3+i)-1,2*(num_t_3+i)-1) = measurement_units(i,6);
    W(2*(num_t_3+i),2*(num_t_3+i)) = measurement_units(i,6);
end
for i = 1:num_t_2 %Confidence for PMU current values
    W(2*(num_t_3+num_t_1+i)-1,2*(num_t_3+num_t_1+i)-1) = measurement_units(num_t_1 + i,6);
    W(2*(num_t_3+num_t_1+i),2*(num_t_3+num_t_1+i)) = measurement_units(num_t_1 + i,6);
end


% Creating empty M-matrix
M = zeros(2 * ( num_t_1 + num_t_2 + num_t_3 ),1);

% Filling M-matrix with measurement data
for i = 1:num_t_3 %SE voltage values
    M(2 * i - 1) = real(measurement_units(num_t_1+ num_t_2 + i,5));
    M(2 * i) = imag(measurement_units(num_t_1+ num_t_2 + i,5));
end
for i = 1:num_t_1 %PMU voltage values
    M(2 * (num_t_3 + i) - 1) = real(measurement_units(i,5));
    M(2 * (num_t_3 + i)) = imag(measurement_units(i,5));
end
for i = 1:num_t_2 %PMU current values
    M(2 * (num_t_3 + num_t_1 + i) - 1) = real(measurement_units(num_t_1 + i,5));
    M(2 * (num_t_3 + num_t_1 + i)) = imag(measurement_units(num_t_1 + i,5));
end



end

